JOIN = ' JOIN {} on {} = {} ', 1
CONDITION = 'WHERE {}', 2
ORDER_BY = ' ORDER BY {}', 3
ORDER_BY_ASC = ' ORDER BY {} ASC ', 3
ORDER_BY_DESC = ' ORDER BY {} DESC ', 3
GROUP_BY = ' GROUP BY {} ', 4
LIMIT = ' LIMIT {} ', 5
