SELECT = 'SELECT COLUMNS_NAME FROM `TABLE_NAME` '

UPDATE = 'UPDATE `TABLE_NAME` SET {} WHERE {} '
INSERT = 'INSERT INTO `TABLE_NAME` ({}) VALUES ({})'
DELETE = 'DELETE FROM `TABLE_NAME` WHERE {}'

SUM = 'SUM({}) as {}'
COUNT = 'COUNT({}) as {}'
MAX = 'MAX({}) as {}'
MIN = 'MIN({}) as {}'
AVG = 'AVG({}) as {}'

WHERE = '{} {} {}'